@@ -48,7 +48,7 @@ static xmlHashTablePtr xmlPredefinedEntities = NULL;
 static void xmlFreeEntity(xmlEntityPtr entity) {
     if (entity == NULL) return;
 
-    if ((entity->children) &&
+    if ((entity->children) && (entity->owner == 1) &&
 	(entity == (xmlEntityPtr) entity->children->parent))
 	xmlFreeNodeList(entity->children);
     if (entity->name != NULL)
@@ -127,6 +127,7 @@ xmlAddEntity(xmlDtdPtr dtd, const xmlChar *name, int type,
     ret->URI = NULL; /* to be computed by the layer knowing
 			the defining entity */
     ret->orig = NULL;
+    ret->owner = 0;
 
     if (xmlHashAddEntry(table, name, ret)) {
 	/*
@@ -419,9 +420,10 @@ static xmlChar *static_buffer = NULL;
 
 static int growBuffer(void) {
     static_buffer_size *= 2;
-    static_buffer = (xmlChar *) xmlRealloc(static_buffer, static_buffer_size * sizeof(xmlChar));
+    static_buffer = (xmlChar *) xmlRealloc(static_buffer,
+	                                static_buffer_size * sizeof(xmlChar));
     if (static_buffer == NULL) {
-        perror("realloc failed");
+        xmlGenericError(xmlGenericErrorContext, "malloc failed\n");
 	return(-1);
     }
     return(0);
@@ -466,9 +468,10 @@ xmlEncodeEntities(xmlDocPtr doc, const xmlChar *input) {
 
     if (static_buffer == NULL) {
         static_buffer_size = 1000;
-        static_buffer = (xmlChar *) xmlMalloc(static_buffer_size * sizeof(xmlChar));
+        static_buffer = (xmlChar *)
+	    xmlMalloc(static_buffer_size * sizeof(xmlChar));
 	if (static_buffer == NULL) {
-	    perror("malloc failed");
+	    xmlGenericError(xmlGenericErrorContext, "malloc failed\n");
             return(NULL);
 	}
 	out = static_buffer;
@@ -561,7 +564,7 @@ xmlEncodeEntities(xmlDocPtr doc, const xmlChar *input) {
     buffer = (xmlChar *)						\
     		xmlRealloc(buffer, buffer_size * sizeof(xmlChar));	\
     if (buffer == NULL) {						\
-	perror("realloc failed");					\
+	xmlGenericError(xmlGenericErrorContext, "realloc failed\n");	\
 	return(NULL);							\
     }									\
 }
@@ -597,7 +600,7 @@ xmlEncodeEntitiesReentrant(xmlDocPtr doc, const xmlChar *input) {
     buffer_size = 1000;
     buffer = (xmlChar *) xmlMalloc(buffer_size * sizeof(xmlChar));
     if (buffer == NULL) {
-	perror("malloc failed");
+	xmlGenericError(xmlGenericErrorContext, "malloc failed\n");
 	return(NULL);
     }
     out = buffer;
@@ -780,7 +783,7 @@ xmlEncodeSpecialChars(xmlDocPtr doc, const xmlChar *input) {
     buffer_size = 1000;
     buffer = (xmlChar *) xmlMalloc(buffer_size * sizeof(xmlChar));
     if (buffer == NULL) {
-	perror("malloc failed");
+	xmlGenericError(xmlGenericErrorContext, "malloc failed\n");
 	return(NULL);
     }
     out = buffer;
@@ -844,6 +847,20 @@ xmlCreateEntitiesTable(void) {
     return((xmlEntitiesTablePtr) xmlHashCreate(0));
 }
 
+/**
+ * xmlFreeEntityWrapper:
+ * @entity:  An entity
+ * @name:  its name
+ *
+ * Deallocate the memory used by an entities in the hash table.
+ */
+static void
+xmlFreeEntityWrapper(xmlEntityPtr entity,
+	               const xmlChar *name ATTRIBUTE_UNUSED) {
+    if (entity != NULL)
+	xmlFreeEntity(entity);
+}
+
 /**
  * xmlFreeEntitiesTable:
  * @table:  An entity table
@@ -852,7 +869,7 @@ xmlCreateEntitiesTable(void) {
  */
 void
 xmlFreeEntitiesTable(xmlEntitiesTablePtr table) {
-    xmlHashFree(table, (xmlHashDeallocator) xmlFreeEntity);
+    xmlHashFree(table, (xmlHashDeallocator) xmlFreeEntityWrapper);
 }
 
 /**
