@@ -240,7 +240,7 @@ struct _xmlElementContent {
     struct _xmlElementContent *c1;	/* first child */
     struct _xmlElementContent *c2;	/* second child */
     struct _xmlElementContent *parent;	/* parent */
-    const xmlChar             *prefix;	/* Element name */
+    const xmlChar             *prefix;	/* Namespace prefix */
 };
 
 /**
@@ -387,6 +387,8 @@ struct _xmlID {
     struct _xmlID    *next;	/* next ID */
     const xmlChar    *value;	/* The ID name */
     xmlAttrPtr        attr;	/* The attribute holding it */
+    const xmlChar    *name;	/* The attribute if attr is not available */
+    int               lineno;	/* The line number if attr is not available */
 };
 
 /**
@@ -401,6 +403,8 @@ struct _xmlRef {
     struct _xmlRef    *next;	/* next Ref */
     const xmlChar     *value;	/* The Ref name */
     xmlAttrPtr        attr;	/* The attribute holding it */
+    const xmlChar    *name;	/* The attribute if attr is not available */
+    int               lineno;	/* The line number if attr is not available */
 };
 
 /**
@@ -839,7 +843,7 @@ int		xmlSaveFile		(const char *filename,
 int		xmlSaveFormatFile	(const char *filename,
 					 xmlDocPtr cur,
 					 int format);
-void		xmlNodeDump		(xmlBufferPtr buf,
+int		xmlNodeDump		(xmlBufferPtr buf,
 					 xmlDocPtr doc,
 					 xmlNodePtr cur,
 					 int level,
@@ -868,6 +872,12 @@ int		xmlSaveFileEnc		(const char *filename,
 					 xmlDocPtr cur,
 					 const char *encoding);
 
+/*
+ * XHTML
+ */
+int		xmlIsXHTML		(const xmlChar *systemID,
+					 const xmlChar *publicID);
+
 /*
  * Compression.
  */
