@@ -74,7 +74,7 @@ class SlowParser(xmllib.XMLParser):
 def getparser(target = None):
     # get the fastest available parser, and attach it to an
     # unmarshalling object.  return both objects.
-    if target == None:
+    if target is None:
         target = docParser()
     if FastParser:
         return FastParser(target), target
@@ -268,6 +268,10 @@ py_types = {
     'xmlCatalogPtr': ('O', "catalog", "xmlCatalogPtr", "xmlCatalogPtr"),
     'FILE *': ('O', "File", "FILEPtr", "FILE *"),
     'xmlURIPtr': ('O', "URI", "xmlURIPtr", "xmlURIPtr"),
+    'xmlOutputBufferPtr': ('O', "outputBuffer", "xmlOutputBufferPtr", "xmlOutputBufferPtr"),
+    'xmlParserInputBufferPtr': ('O', "inputBuffer", "xmlParserInputBufferPtr", "xmlParserInputBufferPtr"),
+    'xmlRegexpPtr': ('O', "xmlReg", "xmlRegexpPtr", "xmlRegexpPtr"),
+    'xmlTextReaderPtr': ('O', "xmlTextReader", "xmlTextReaderPtr", "xmlTextReaderPtr"),
 }
 
 py_return_types = {
@@ -383,6 +387,35 @@ def print_function_wrapper(name, output, export, include):
             unknown_types[ret[0]] = [name]
         return -1
 
+    if file == "debugXML":
+        include.write("#ifdef LIBXML_DEBUG_ENABLED\n");
+        export.write("#ifdef LIBXML_DEBUG_ENABLED\n");
+        output.write("#ifdef LIBXML_DEBUG_ENABLED\n");
+    elif file == "HTMLtree" or file == "HTMLparser":
+        include.write("#ifdef LIBXML_HTML_ENABLED\n");
+        export.write("#ifdef LIBXML_HTML_ENABLED\n");
+        output.write("#ifdef LIBXML_HTML_ENABLED\n");
+    elif file == "c14n":
+        include.write("#ifdef LIBXML_C14N_ENABLED\n");
+        export.write("#ifdef LIBXML_C14N_ENABLED\n");
+        output.write("#ifdef LIBXML_C14N_ENABLED\n");
+    elif file == "xpathInternals" or file == "xpath":
+        include.write("#ifdef LIBXML_XPATH_ENABLED\n");
+        export.write("#ifdef LIBXML_XPATH_ENABLED\n");
+        output.write("#ifdef LIBXML_XPATH_ENABLED\n");
+    elif file == "xpointer":
+        include.write("#ifdef LIBXML_XPTR_ENABLED\n");
+        export.write("#ifdef LIBXML_XPTR_ENABLED\n");
+        output.write("#ifdef LIBXML_XPTR_ENABLED\n");
+    elif file == "xinclude":
+        include.write("#ifdef LIBXML_XINCLUDE_ENABLED\n");
+        export.write("#ifdef LIBXML_XINCLUDE_ENABLED\n");
+        output.write("#ifdef LIBXML_XINCLUDE_ENABLED\n");
+    elif file == "xmlregexp":
+        include.write("#ifdef LIBXML_REGEXP_ENABLED\n");
+        export.write("#ifdef LIBXML_REGEXP_ENABLED\n");
+        output.write("#ifdef LIBXML_REGEXP_ENABLED\n");
+
     include.write("PyObject * ")
     include.write("libxml_%s(PyObject *self, PyObject *args);\n" % (name));
 
@@ -392,7 +425,7 @@ def print_function_wrapper(name, output, export, include):
     if file == "python":
         # Those have been manually generated
         return 1
-    if file == "python_accessor" and ret[0] != "void" and ret[2] == None:
+    if file == "python_accessor" and ret[0] != "void" and ret[2] is None:
         # Those have been manually generated
         return 1
 
@@ -417,6 +450,34 @@ def print_function_wrapper(name, output, export, include):
     output.write(c_call)
     output.write(ret_convert)
     output.write("}\n\n")
+    if file == "debugXML":
+        include.write("#endif /* LIBXML_DEBUG_ENABLED */\n");
+        export.write("#endif /* LIBXML_DEBUG_ENABLED */\n");
+        output.write("#endif /* LIBXML_DEBUG_ENABLED */\n");
+    elif file == "HTMLtree" or file == "HTMLparser":
+        include.write("#endif /* LIBXML_HTML_ENABLED */\n");
+        export.write("#endif /* LIBXML_HTML_ENABLED */\n");
+        output.write("#endif /* LIBXML_HTML_ENABLED */\n");
+    elif file == "c14n":
+        include.write("#endif /* LIBXML_C14N_ENABLED */\n");
+        export.write("#endif /* LIBXML_C14N_ENABLED */\n");
+        output.write("#endif /* LIBXML_C14N_ENABLED */\n");
+    elif file == "xpathInternals" or file == "xpath":
+        include.write("#endif /* LIBXML_XPATH_ENABLED */\n");
+        export.write("#endif /* LIBXML_XPATH_ENABLED */\n");
+        output.write("#endif /* LIBXML_XPATH_ENABLED */\n");
+    elif file == "xpointer":
+        include.write("#endif /* LIBXML_XPTR_ENABLED */\n");
+        export.write("#endif /* LIBXML_XPTR_ENABLED */\n");
+        output.write("#endif /* LIBXML_XPTR_ENABLED */\n");
+    elif file == "xinclude":
+        include.write("#endif /* LIBXML_XINCLUDE_ENABLED */\n");
+        export.write("#endif /* LIBXML_XINCLUDE_ENABLED */\n");
+        output.write("#endif /* LIBXML_XINCLUDE_ENABLED */\n");
+    elif file == "xmlregexp":
+        include.write("#endif /* LIBXML_REGEXP_ENABLED */\n");
+        export.write("#endif /* LIBXML_REGEXP_ENABLED */\n");
+        output.write("#endif /* LIBXML_REGEXP_ENABLED */\n");
     return 1
 
 def buildStubs():
@@ -468,7 +529,7 @@ def buildStubs():
     wrapper = open("libxml2-py.c", "w")
     wrapper.write("/* Generated */\n\n")
     wrapper.write("#include <Python.h>\n")
-    wrapper.write("#include \"config.h\"\n")
+#    wrapper.write("#include \"config.h\"\n")
     wrapper.write("#include <libxml/xmlversion.h>\n")
     wrapper.write("#include <libxml/tree.h>\n")
     wrapper.write("#include \"libxml_wrap.h\"\n")
@@ -534,6 +595,10 @@ classes_type = {
     "htmlParserCtxt *": ("._o", "parserCtxt(_obj=%s)", "parserCtxt"),
     "xmlCatalogPtr": ("._o", "catalog(_obj=%s)", "catalog"),
     "xmlURIPtr": ("._o", "URI(_obj=%s)", "URI"),
+    "xmlOutputBufferPtr": ("._o", "outputBuffer(_obj=%s)", "outputBuffer"),
+    "xmlParserInputBufferPtr": ("._o", "inputBuffer(_obj=%s)", "inputBuffer"),
+    "xmlRegexpPtr": ("._o", "xmlReg(_obj=%s)", "xmlReg"),
+    "xmlTextReaderPtr": ("._o", "xmlTextReader(_obj=%s)", "xmlTextReader"),
 }
 
 converter_type = {
@@ -551,11 +616,17 @@ classes_ancestor = {
     "xmlEntity" : "xmlNode",
     "xmlElement" : "xmlNode",
     "xmlAttribute" : "xmlNode",
+    "outputBuffer": "ioWriteWrapper",
+    "inputBuffer": "ioReadWrapper",
 }
 classes_destructors = {
     "parserCtxt": "xmlFreeParserCtxt",
     "catalog": "xmlFreeCatalog",
     "URI": "xmlFreeURI",
+#    "outputBuffer": "xmlOutputBufferClose",
+    "inputBuffer": "xmlFreeParserInputBuffer",
+    "xmlReg": "xmlRegFreeRegexp",
+    "xmlTextReader": "xmlFreeTextReader",
 }
 
 functions_noexcept = {
@@ -563,6 +634,10 @@ functions_noexcept = {
     "xmlHasNsProp": 1,
 }
 
+reference_keepers = {
+    "xmlTextReader": [('inputBuffer', 'input')],
+}
+
 function_classes = {}
 
 function_classes["None"] = []
@@ -598,6 +673,18 @@ def nameFixup(name, classe, type, file):
     elif name[0:11] == "xmlXPathSet" and file == "python_accessor":
         func = name[8:]
         func = string.lower(func[0:1]) + func[1:]
+    elif name[0:15] == "xmlOutputBuffer" and file != "python":
+        func = name[15:]
+        func = string.lower(func[0:1]) + func[1:]
+    elif name[0:20] == "xmlParserInputBuffer" and file != "python":
+        func = name[20:]
+        func = string.lower(func[0:1]) + func[1:]
+    elif name[0:9] == "xmlRegexp" and file == "xmlregexp":
+        func = "regexp" + name[9:]
+    elif name[0:6] == "xmlReg" and file == "xmlregexp":
+        func = "regexp" + name[6:]
+    elif name[0:13] == "xmlTextReader" and file == "xmlreader":
+        func = name[13:]
     elif name[0:11] == "xmlACatalog":
         func = name[11:]
         func = string.lower(func[0:1]) + func[1:]
@@ -651,7 +738,7 @@ def functionCompare(info1, info2):
     return 0
 
 def writeDoc(name, args, indent, output):
-     if functions[name][0] == None or functions[name][0] == "":
+     if functions[name][0] is None or functions[name][0] == "":
          return
      val = functions[name][0]
      val = string.replace(val, "NULL", "None");
@@ -668,7 +755,7 @@ def writeDoc(name, args, indent, output):
          output.write('\n  ');
          output.write(indent)
      output.write(val);
-     output.write('"""\n')
+     output.write(' """\n')
 
 def buildWrappers():
     global ctypes
@@ -789,7 +876,7 @@ def buildWrappers():
 
 	    for arg in args:
 		if classes_type.has_key(arg[1]):
-		    classes.write("    if %s == None: %s__o = None\n" %
+		    classes.write("    if %s is None: %s__o = None\n" %
 				  (arg[0], arg[0]))
 		    classes.write("    else: %s__o = %s%s\n" %
 				  (arg[0], arg[0], classes_type[arg[1]][0]))
@@ -813,22 +900,22 @@ def buildWrappers():
 		    # Raise an exception
 		    #
 		    if functions_noexcept.has_key(name):
-		        classes.write("    if ret == None:return None\n");
+		        classes.write("    if ret is None:return None\n");
 		    elif string.find(name, "URI") >= 0:
 			classes.write(
-			"    if ret == None:raise uriError('%s() failed')\n"
+			"    if ret is None:raise uriError('%s() failed')\n"
 			              % (name))
 		    elif string.find(name, "XPath") >= 0:
 			classes.write(
-			"    if ret == None:raise xpathError('%s() failed')\n"
+			"    if ret is None:raise xpathError('%s() failed')\n"
 			              % (name))
 		    elif string.find(name, "Parse") >= 0:
 			classes.write(
-			"    if ret == None:raise parserError('%s() failed')\n"
+			"    if ret is None:raise parserError('%s() failed')\n"
 			              % (name))
 		    else:
 			classes.write(
-			"    if ret == None:raise treeError('%s() failed')\n"
+			"    if ret is None:raise treeError('%s() failed')\n"
 			              % (name))
 		    classes.write("    return ");
 		    classes.write(classes_type[ret[0]][1] % ("ret"));
@@ -848,6 +935,10 @@ def buildWrappers():
 		classes.write("class %s(%s):\n" % (classname,
 			      classes_ancestor[classname]))
 		classes.write("    def __init__(self, _obj=None):\n")
+		if reference_keepers.has_key(classname):
+		    rlist = reference_keepers[classname]
+		    for ref in rlist:
+		        classes.write("        self.%s = None\n" % ref[1])
 		classes.write("        self._o = None\n")
 		classes.write("        %s.__init__(self, _obj=_obj)\n\n" % (
 			      classes_ancestor[classname]))
@@ -861,6 +952,10 @@ def buildWrappers():
 		txt.write("Class %s()\n" % (classname))
 		classes.write("class %s:\n" % (classname))
 		classes.write("    def __init__(self, _obj=None):\n")
+		if reference_keepers.has_key(classname):
+		    list = reference_keepers[classname]
+		    for ref in list:
+		        classes.write("        self.%s = None\n" % ref[1])
 		classes.write("        if _obj != None:self._o = _obj;return\n")
 		classes.write("        self._o = None\n\n");
 	    if classes_destructors.has_key(classname):
@@ -898,7 +993,7 @@ def buildWrappers():
 		for arg in args:
 		    if classes_type.has_key(arg[1]):
 			if n != index:
-			    classes.write("        if %s == None: %s__o = None\n" %
+			    classes.write("        if %s is None: %s__o = None\n" %
 					  (arg[0], arg[0]))
 			    classes.write("        else: %s__o = %s%s\n" %
 					  (arg[0], arg[0], classes_type[arg[1]][0]))
@@ -929,48 +1024,69 @@ def buildWrappers():
 			#
 			if functions_noexcept.has_key(name):
 			    classes.write(
-			        "        if ret == None:return None\n");
+			        "        if ret is None:return None\n");
 			elif string.find(name, "URI") >= 0:
 			    classes.write(
-		    "        if ret == None:raise uriError('%s() failed')\n"
+		    "        if ret is None:raise uriError('%s() failed')\n"
 					  % (name))
 			elif string.find(name, "XPath") >= 0:
 			    classes.write(
-		    "        if ret == None:raise xpathError('%s() failed')\n"
+		    "        if ret is None:raise xpathError('%s() failed')\n"
 					  % (name))
 			elif string.find(name, "Parse") >= 0:
 			    classes.write(
-		    "        if ret == None:raise parserError('%s() failed')\n"
+		    "        if ret is None:raise parserError('%s() failed')\n"
 					  % (name))
 			else:
 			    classes.write(
-		    "        if ret == None:raise treeError('%s() failed')\n"
+		    "        if ret is None:raise treeError('%s() failed')\n"
 					  % (name))
-			classes.write("        return ");
+
+			#
+			# generate the returned class wrapper for the object
+			#
+			classes.write("        __tmp = ");
 			classes.write(classes_type[ret[0]][1] % ("ret"));
 			classes.write("\n");
+
+                        #
+			# Sometime one need to keep references of the source
+			# class in the returned class object.
+			# See reference_keepers for the list
+			#
+			tclass = classes_type[ret[0]][2]
+			if reference_keepers.has_key(tclass):
+			    list = reference_keepers[tclass]
+			    for pref in list:
+				if pref[0] == ref[0]:
+				    classes.write("        __tmp.%s = self\n" %
+						  pref[1])
+			#
+			# return the class
+			#
+			classes.write("        return __tmp\n");
 		    elif converter_type.has_key(ret[0]):
 			#
 			# Raise an exception
 			#
 			if functions_noexcept.has_key(name):
 			    classes.write(
-			        "        if ret == None:return None");
+			        "        if ret is None:return None");
 			elif string.find(name, "URI") >= 0:
 			    classes.write(
-		    "        if ret == None:raise uriError('%s() failed')\n"
+		    "        if ret is None:raise uriError('%s() failed')\n"
 					  % (name))
 			elif string.find(name, "XPath") >= 0:
 			    classes.write(
-		    "        if ret == None:raise xpathError('%s() failed')\n"
+		    "        if ret is None:raise xpathError('%s() failed')\n"
 					  % (name))
 			elif string.find(name, "Parse") >= 0:
 			    classes.write(
-		    "        if ret == None:raise parserError('%s() failed')\n"
+		    "        if ret is None:raise parserError('%s() failed')\n"
 					  % (name))
 			else:
 			    classes.write(
-		    "        if ret == None:raise treeError('%s() failed')\n"
+		    "        if ret is None:raise treeError('%s() failed')\n"
 					  % (name))
 			classes.write("        return ");
 			classes.write(converter_type[ret[0]] % ("ret"));
