@@ -135,7 +135,7 @@ static int xmlOutputCallbackInitialized = 0;
 
 
 /**
- * xmlNormalizeWindowsPath
+ * xmlNormalizeWindowsPath:
  * @path:  a windows path like "C:/foo/bar"
  *
  * Normalize a Windows path to make an URL from it
@@ -246,7 +246,7 @@ xmlCleanupOutputCallbacks(void)
  ************************************************************************/
 
 /**
- * xmlCheckFilename
+ * xmlCheckFilename:
  * @path:  the path to check
  *
  * function checks to see if @path is a valid source
@@ -453,7 +453,11 @@ xmlFileRead (void * context, char * buffer, int len) {
  */
 static int
 xmlFileWrite (void * context, const char * buffer, int len) {
-    return(fwrite(&buffer[0], 1,  len, (FILE *) context));
+    int items;
+
+    items = fwrite(&buffer[0], len, 1, (FILE *) context);
+
+    return(items * len);
 }
 
 /**
@@ -461,6 +465,8 @@ xmlFileWrite (void * context, const char * buffer, int len) {
  * @context:  the I/O context
  *
  * Close an I/O channel
+ *
+ * Returns 0 or -1 in case of error
  */
 int
 xmlFileClose (void * context) {
@@ -1033,7 +1039,7 @@ xmlIOHTTPOpen (const char *filename) {
 }
 
 /**
- * xmlIOHTTPOpenW
+ * xmlIOHTTPOpenW:
  * @post_uri:  The destination URI for the document
  * @compression:  The compression desired for the document.
  *
@@ -1176,6 +1182,8 @@ xmlIOHTTPWrite( void * context, const char * buffer, int len ) {
  * @context:  the I/O context
  *
  * Close an HTTP I/O channel
+ *
+ * Returns 0
  */
 int
 xmlIOHTTPClose (void * context) {
@@ -1396,6 +1404,8 @@ xmlIOFTPRead(void * context, char * buffer, int len) {
  * @context:  the I/O context
  *
  * Close an FTP I/O channel
+ *
+ * Returns 0
  */
 int
 xmlIOFTPClose (void * context) {
@@ -1524,7 +1534,7 @@ xmlRegisterDefaultOutputCallbacks
 
 #ifdef LIBXML_HTTP_ENABLED
 /**
- * xmlRegisterHTTPPostCallbacks
+ * xmlRegisterHTTPPostCallbacks:
  *
  * By default, libxml submits HTTP output requests using the "PUT" method.
  * Calling this method changes the HTTP output method to use the "POST"
