@@ -86,10 +86,10 @@
 #include <libxml/xmlreader.h>
 
 #ifdef LIBXML_DEBUG_ENABLED
-static int debug = 0;
 static int shell = 0;
 static int debugent = 0;
 #endif
+static int debug = 0;
 static int copy = 0;
 static int recovery = 0;
 static int noent = 0;
@@ -510,6 +510,7 @@ xmlHTMLValidityWarning(void *ctx, const char *msg, ...)
  * 			Shell Interface					*
  * 									*
  ************************************************************************/
+#ifdef LIBXML_DEBUG_ENABLED
 /**
  * xmlShellReadline:
  * @prompt:  the prompt value
@@ -550,6 +551,7 @@ xmlShellReadline(char *prompt) {
     return(ret);
 #endif
 }
+#endif /* LIBXML_DEBUG_ENABLED */
 
 /************************************************************************
  * 									*
@@ -1132,6 +1134,8 @@ static void usage(const char *name) {
     printf("\t--debug : dump a debug tree of the in-memory document\n");
     printf("\t--shell : run a navigating shell\n");
     printf("\t--debugent : debug the entities defined in the document\n");
+#else
+    printf("\t--debug : dump the nodes content when using --stream\n");
 #endif
     printf("\t--copy : used to test the internal copy implementation\n");
     printf("\t--recover : output what was parsable on broken XML documents\n");
@@ -1213,10 +1217,11 @@ main(int argc, char **argv) {
 
 	if (argv[i][0] != '-')
 	    continue;
-#ifdef LIBXML_DEBUG_ENABLED
 	if ((!strcmp(argv[i], "-debug")) || (!strcmp(argv[i], "--debug")))
 	    debug++;
-	else if ((!strcmp(argv[i], "-shell")) ||
+	else
+#ifdef LIBXML_DEBUG_ENABLED
+	if ((!strcmp(argv[i], "-shell")) ||
 	         (!strcmp(argv[i], "--shell"))) {
 	    shell++;
             noout = 1;
